package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.StreamCipher;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class VMPCEngine implements StreamCipher {
   protected byte n = 0;
   protected byte[] P = null;
   protected byte s = 0;
   protected byte[] workingIV;
   protected byte[] workingKey;

   public String getAlgorithmName() {
      return "VMPC";
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof ParametersWithIV) {
         ParametersWithIV var10000 = (ParametersWithIV)params;
         ParametersWithIV forEncryption1;
         KeyParameter params1 = (KeyParameter)(forEncryption1 = (ParametersWithIV)params).getParameters();
         if (var10000.getParameters() instanceof KeyParameter) {
            byte[] forEncryption2;
            byte[] var7 = forEncryption2 = forEncryption1.getIV();
            this.workingIV = forEncryption2;
            if (var7 != null && forEncryption2.length >= 1 && forEncryption2.length <= 768) {
               VMPCEngine var8 = this;
               VMPCEngine var10001 = this;
               byte[] var3;
               this.workingKey = var3 = params1.getKey();
               var8.initKey(var3, var10001.workingIV);
            } else {
               throw new IllegalArgumentException("VMPC requires 1 to 768 bytes of IV");
            }
         } else {
            throw new IllegalArgumentException("VMPC init parameters must include a key");
         }
      } else {
         throw new IllegalArgumentException("VMPC init parameters must include an IV");
      }
   }

   protected void initKey(byte[] keyBytes, byte[] ivBytes) {
      this.s = 0;
      this.P = new byte[256];

      int var3;
      for(var3 = 0; var3 < 256; ++var3) {
         this.P[var3] = (byte)var3;
      }

      int var8;
      byte[] var10000;
      byte var10001;
      for(var3 = 0; var3 < 768; ++var3) {
         byte[] var4;
         var10000 = var4 = this.P;
         int var5;
         byte var6;
         var10001 = var6 = var10000[this.s + var4[var5 = var3 & 255] + keyBytes[var3 % keyBytes.length] & 255];
         this.s = var6;
         var8 = var4[var5];
         var4[var5] = var4[var6 & 255];
         var10000[var10001 & 255] = (byte)var8;
      }

      for(int keyBytes1 = 0; keyBytes1 < 768; ++keyBytes1) {
         byte[] var9;
         var10000 = var9 = this.P;
         byte var11;
         var10001 = var11 = var10000[this.s + var9[var8 = keyBytes1 & 255] + ivBytes[keyBytes1 % ivBytes.length] & 255];
         this.s = var11;
         byte var10 = var9[var8];
         var9[var8] = var9[var11 & 255];
         var10000[var10001 & 255] = var10;
      }

      this.n = 0;
   }

   public void processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      if (inOff + len > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + len > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         for(int var6 = 0; var6 < len; ++var6) {
            byte[] var7 = this.P;
            int var8 = this.s;
            int var9;
            byte var10007 = (byte)(var9 = this.n);
            byte var10009 = (byte)(var8 = var7[var8 + var7[var9 & 255] & 255]);
            byte var10012 = this.s = (byte)var8;
            byte var10 = var7[var7[var7[var8 & 255] & 255] + 1 & 255];
            var8 = var7[var9 & 255];
            var9 &= 255;
            var7[var9] = var7[var10012 & 255];
            var7[var10009 & 255] = (byte)var8;
            this.n = (byte)(var10007 + 1 & 255);
            var8 = var6 + outOff;
            out[var8] = (byte)(in[var6 + inOff] ^ var10);
         }

      }
   }

   public void reset() {
      VMPCEngine var10000 = this;
      VMPCEngine var10001 = this;
      byte[] var1 = this.workingKey;
      var10000.initKey(var1, var10001.workingIV);
   }

   public byte returnByte(byte in) {
      byte var10000 = in;
      VMPCEngine var10001 = this;
      byte[] in1 = this.P;
      int var2 = this.s;
      byte var3;
      byte var10002 = var3 = this.n;
      byte[] var10003 = in1;
      byte var10004 = (byte)(var2 = in1[var2 + in1[var3 & 255] & 255]);
      byte[] var10005 = in1;
      byte[] var10006 = in1;
      byte var10007 = this.s = (byte)var2;
      byte var4 = in1[in1[in1[var2 & 255] & 255] + 1 & 255];
      in = in1[var3 & 255];
      var2 = var3 & 255;
      var10005[var2] = var10006[var10007 & 255];
      var10003[var10004 & 255] = in;
      var10001.n = (byte)(var10002 + 1 & 255);
      return (byte)(var10000 ^ var4);
   }
}
